@@ -24,6 +24,7 @@ from .forms import (
     SampleForm2,
     SampleForm3,
     SampleForm4,
+    SampleForm5,
     SampleForm6,
     SelectSampleForm,
 )
@@ -585,32 +586,79 @@ def test_keepcontext_context_manager(settings):
 def test_use_custom_control_is_used_in_checkboxes():
     form = CheckboxesSampleForm()
     form.helper = FormHelper()
-    form.helper.layout = Layout("checkboxes", InlineCheckboxes("alphacheckboxes"), "numeric_multiple_checkboxes")
+    form.helper.layout = Layout(
+        "checkboxes",
+        InlineCheckboxes("alphacheckboxes"),
+        "numeric_multiple_checkboxes",
+    )
     # form.helper.use_custom_control take default value which is True
-
-    response = render(request=None, template_name="crispy_render_template.html", context={"form": form})
-    assert response.content.count(b"custom-control-inline") == 3
-    assert response.content.count(b"custom-checkbox") == 9
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_checkboxes_true.html"
+    )
 
     form.helper.use_custom_control = True
-
-    response = render(request=None, template_name="crispy_render_template.html", context={"form": form})
-    assert response.content.count(b"custom-control-inline") == 3
-    assert response.content.count(b"custom-checkbox") == 9
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_checkboxes_true.html"
+    )
 
     form.helper.use_custom_control = False
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_checkboxes_false.html"
+    )
 
-    response = render(request=None, template_name="crispy_render_template.html", context={"form": form})
-
-    assert response.content.count(b"custom-control-inline") == 0
-    assert response.content.count(b"form-check-inline") == 3
-    assert response.content.count(b"form-check") > 0
-    assert response.content.count(b"custom-checkbox") == 0
+    form = CheckboxesSampleForm({})
+    form.helper = FormHelper()
+    form.helper.layout = Layout(
+        "checkboxes",
+        InlineCheckboxes("alphacheckboxes"),
+        "numeric_multiple_checkboxes",
+    )
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_checkboxes_true_failing.html"
+    )
 
 
 @only_bootstrap4
-@pytest.mark.parametrize("use_custom_control, custom_select_count", [(True, 1), (False, 0)])
-def test_use_custom_control_in_select(use_custom_control, custom_select_count):
+def test_use_custom_control_is_used_in_radio():
+    form = SampleForm5()
+    form.helper = FormHelper()
+    form.helper.layout = Layout(
+        "radio_select",
+    )
+    # form.helper.use_custom_control take default value which is True
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_true.html"
+    )
+
+    form.helper.use_custom_control = True
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_true.html"
+    )
+
+    form.helper.use_custom_control = False
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_false.html"
+    )
+
+    form = SampleForm5({})
+    form.helper = FormHelper()
+    form.helper.layout = Layout(
+        "radio_select",
+    )
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_true_failing.html"
+    )
+
+
+@only_bootstrap4
+@pytest.mark.parametrize(
+    "use_custom_control, expected_html",
+    [
+        (True, "bootstrap4/test_layout/test_use_custom_control_in_select_true.html"),
+        (False, "bootstrap4/test_layout/test_use_custom_control_in_select_false.html"),
+    ],
+)
+def test_use_custom_control_in_select(use_custom_control, expected_html):
     form = SelectSampleForm()
 
     form.helper = FormHelper()
@@ -618,9 +666,7 @@ def test_use_custom_control_in_select(use_custom_control, custom_select_count):
     form.helper.layout = Layout("select")
     form.helper.use_custom_control = use_custom_control
 
-    html = render_crispy_form(form)
-
-    assert html.count("custom-select") == custom_select_count
+    assert parse_form(form) == parse_expected(expected_html)
 
 
 @only_bootstrap3
@@ -684,17 +730,17 @@ def test_file_field():
     form = FileForm()
     form.helper = FormHelper()
     form.helper.layout = Layout("clearable_file")
-    assert parse_form(form) == parse_expected("bootstrap4/file_field/clearable_file_field_custom_control.html")
+    assert parse_form(form) == parse_expected("bootstrap4/test_layout/test_file_field_clearable_custom_control.html")
 
     form.helper.use_custom_control = False
-    assert parse_form(form) == parse_expected("bootstrap4/file_field/clearable_file_field.html")
+    assert parse_form(form) == parse_expected("bootstrap4/test_layout/test_file_field_clearable.html")
 
     form.helper.use_custom_control = True
     form.helper.layout = Layout("file_field")
-    assert parse_form(form) == parse_expected("bootstrap4/file_field/file_field_custom_control.html")
+    assert parse_form(form) == parse_expected("bootstrap4/test_layout/test_file_field_custom_control.html")
 
     form.helper.use_custom_control = False
-    assert parse_form(form) == parse_expected("bootstrap4/file_field/file_field.html")
+    assert parse_form(form) == parse_expected("bootstrap4/test_layout/test_file_field_default.html")
 
 
 @only_bootstrap4
@@ -702,7 +748,9 @@ def test_file_field_with_custom_class():
     form = AdvancedFileForm()
     form.helper = FormHelper()
     form.helper.layout = Layout("clearable_file")
-    assert parse_form(form) == parse_expected("bootstrap4/file_field/clearable_file_field_custom_class.html")
+    assert parse_form(form) == parse_expected(
+        "bootstrap4/test_layout/test_file_field_with_custom_class_clearable.html"
+    )
 
     form.helper.layout = Layout("file_field")
-    assert parse_form(form) == parse_expected("bootstrap4/file_field/file_field_custom_class.html")
+    assert parse_form(form) == parse_expected("bootstrap4/test_layout/test_file_field_with_custom_class.html")
